----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    08:58:03 05/16/2008 
-- Design Name: 
-- Module Name:    float_mult - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity float_mult is
    Port ( A : in  STD_LOGIC_VECTOR (15 downto 0);
           B : in  STD_LOGIC_VECTOR (15 downto 0);
           M : out  STD_LOGIC_VECTOR (15 downto 0));
end float_mult;

architecture arch_floatmult of float_mult is
component adder is
	 generic (width : integer);
    Port ( A : in  STD_LOGIC_VECTOR (width-1 downto 0);
           B : in  STD_LOGIC_VECTOR (width-1 downto 0);
           S : out  STD_LOGIC_VECTOR (width-1 downto 0);
			  cin: in STD_LOGIC;
			  cout: out STD_LOGIC);
end component adder;
component matrix_mult is 
	 generic (width : Integer);
    Port ( X : in  STD_LOGIC_VECTOR (width-1 downto 0);
           Y : in  STD_LOGIC_VECTOR (width-1 downto 0);
           M : out  STD_LOGIC_VECTOR (2*width-1 downto 0));
end component matrix_mult;
signal partialexp : STD_LOGIC_VECTOR (4 downto 0);
signal partialexpb : STD_LOGIC_VECTOR (4 downto 0);
signal amant : STD_LOGIC_VECTOR (10 downto 0);
signal bmant : STD_LOGIC_VECTOR (10 downto 0);
signal multip : STD_LOGIC_VECTOR (21 downto 0);
signal normmultip : STD_LOGIC_VECTOR (21 downto 0);
signal roundmultip : STD_LOGIC_VECTOR (10 downto 0);
signal doround : STD_LOGIC;
signal corectround : STD_LOGIC;
signal final_correction: STD_LOGIC;
signal final_correction_expanded: STD_LOGIC_VECTOR(4 downto 0);
signal final_value: STD_LOGIC_VECTOR(9 downto 0);
begin
	M(15) <= A(15) xor B(15);
	expadder_1: adder generic map(width => 5)
				port map (A=>A(14 downto 10),
				          B=>B(14 downto 10),
							 S=>partialexp,
							 cin => '0');
	expadder_2: adder generic map(width => 5)
				port map (A=> partialexp,
				          B=> "10001",
							 S => partialexpb,
							 cin => '0');
	expadder_3: adder generic map(width => 5)
				port map (A=> partialexpb,
				          B=> (final_correction_expanded),
							 S => M(14 downto 10),
							 cin => corectround);
	amant(9 downto 0) <= A(9 downto 0);
	bmant(9 downto 0) <= B(9 downto 0);
	amant(10) <= '1';
	bmant(10) <= '1';
	mymult : matrix_mult generic map (width => 11)
	    port map ( X=>amant, Y=> bmant, M=> multip);
	normmultip <= multip when multip(21)='1' else
	      multip (20 downto 0) & '0';
	corectround <= multip(21);
	roundadd : adder generic map(width => 10)
	     port map (A => "0000000000",
		            B => normmultip(20 downto 11),
						S => final_value,
						cin => doround,
						cout => final_correction);
	doround <= '1' when (normmultip(11 downto 0) = "110000000000") else
	    '0';
		 
	wordgate: for i in 9 downto 0 generate
		M (i) <= final_value(i) and (not final_correction);
	end generate;
	
	final_correction_expanded <= "0000" & final_correction;
	
end architecture arch_floatmult;

