library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity matrix_mult is
	 generic (width : Integer := 8);
    Port ( X : in  STD_LOGIC_VECTOR (width-1 downto 0);
           Y : in  STD_LOGIC_VECTOR (width-1 downto 0);
           M : out  STD_LOGIC_VECTOR (2*width-1 downto 0));
end matrix_mult;

architecture Behavioral of matrix_mult is
component csa is
	generic (width: integer);
   port ( A : in   STD_LOGIC_VECTOR (width-1 downto 0);
          B : in   STD_LOGIC_VECTOR (width-1 downto 0);
          Cin : in   STD_LOGIC_VECTOR (width-1 downto 0);
          S : out  STD_LOGIC_VECTOR (width-1 downto 0);
          Cout : out  STD_LOGIC_VECTOR (width   downto 1));
end component csa;
component and_wordgate is
	 generic (width: integer);
    Port ( input : in  STD_LOGIC_VECTOR (width-1 downto 0);
           output : out  STD_LOGIC_VECTOR (width-1 downto 0);
           active : in  STD_LOGIC);
end component;
component adder is
	 generic (width : integer);
    Port ( A : in  STD_LOGIC_VECTOR (width-1 downto 0);
           B : in  STD_LOGIC_VECTOR (width-1 downto 0);
           S : out  STD_LOGIC_VECTOR (width-1 downto 0);
			  cin: in STD_LOGIC;
			  cout: out STD_LOGIC);
end component adder;
subtype bitrow is STD_LOGIC_VECTOR(width-1 downto 0);
type bitmatrix is array(width-1 downto 0) of bitrow;
type bigbitmatrix is array(width downto 0) of bitrow;
signal Xsel:bitmatrix;
signal sums, carries, shiftedsums: bigbitmatrix;
signal zero: std_logic;
begin
	zero <= '0';
	wgs : for i in width-1 downto 0 generate
		awg: and_wordgate generic map(width)
			port map(input => X,
						output => Xsel(i),
						active => Y(i));
		carries(0)(i) <= '0';
		sums(0)(i) <= '0';
		shiftedsums(i) <= '0' & sums(i)(width-1 downto 1);
		M(i) <= sums(i+1)(0);
		csa: csa generic map(width)
		   port map(A => Xsel(i),
					   Cin => carries(i),
						B => shiftedsums(i),
						S => sums(i+1),
						Cout => carries(i+1));
	end generate;
	shiftedsums(width) <= '0' & sums(width)(width-1 downto 1);
	finaladder: adder generic map(width)
		port map(A => shiftedsums(width), 
				   B => carries(width),
					S => M(width * 2 -1 downto width),
					cin => '0');
end Behavioral;

